"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ensureDBdir = ensureDBdir;
exports.emptyDBdir = emptyDBdir;
exports.streamDB_ = streamDB_;
exports.streamDB = streamDB;
exports.checkConnection = checkConnection;
exports.queryDBs = queryDBs;
exports.getFlex = getFlex;
exports.getTerms = getTerms;
exports.getDB = getDB;
exports.createDB = createDB;
exports.readDictionary = readDictionary;
exports.updateCurrent = updateCurrent;
exports.delDictionary = delDictionary;

var _lodash = _interopRequireDefault(require("lodash"));

var _verbRegKeys = require("./verb-reg-keys");

var _nameRegKeys = require("./name-reg-keys");

var _orthos = require("orthos");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var path = require('path');

var fse = require('fs-extra');

var PouchDB = require('pouchdb'); // STREAM


var replicationStream = require('pouchdb-replication-stream');

PouchDB.plugin(replicationStream.plugin);
PouchDB.adapter('writableStream', replicationStream.adapters.writableStream);

var MemoryStream = require('memorystream');

var nkeys = _lodash["default"].values(_nameRegKeys.namekeys);

var vkeys = _lodash["default"].values(_verbRegKeys.verbkeys);

// import {oxia, comb, plain} from '../../../../greek/orthos'
var request = require('request');

var rp = require('request-promise');

var log = console.log;
var dbs = [];
var actives = ['flex', 'terms', 'wkt', 'lsj'];
var opts = {
  "host": 'http://diglossa.org:5984',
  code: 'grc',
  json: true
};

function ensureDBdir(upath) {
  var pouchpath = path.resolve(upath, 'pouch');
  fse.ensureDirSync(pouchpath);
  dbs = [];
}

function emptyDBdir(upath) {
  var pouchpath = path.resolve(upath, 'pouch');
  fse.emptyDirSync(pouchpath);
  dbs = [];
}

function streamDB_(upath, dname, stream, batch_size) {
  var pouchpath = path.resolve(upath, 'pouch');
  fse.ensureDirSync(pouchpath);
  var dpath = path.resolve(upath, 'pouch', dname);
  fse.emptyDirSync(dpath);
  var pouch = new PouchDB(dpath);
  pouch.dname = dname;
  var longname = [opts.code, '-', dname].join('');
  var spath = [opts.host, longname].join('/');
  var source = new PouchDB(spath, {
    skip_setup: true
  }); // return pouch.info()

  return Promise.all([source.dump(stream, {
    batch_size: batch_size,
    live: true,
    retry: true
  }), // .catch(err=> {
  //   console.log('ERR: DUMP', dname, spath, err.message)
  // }),
  pouch.load(stream) // .catch(err=> {
  //   console.log('ERR: LOAD', dname, dpath, err.message)
  // })
  ]).then(function (res) {
    dbs.push(pouch); // pouch.close()

    return dname;
  }); // .catch(err=> {
  //   log('ERR: stream dump err', dname, err.message)
  //   // return
  // })
}

function streamDB(upath, dict, step, ocounter) {
  var dname = dict.dname;
  var pouchpath = path.resolve(upath, 'pouch');
  fse.ensureDirSync(pouchpath);
  var dpath = path.resolve(upath, 'pouch', dname);
  fse.emptyDirSync(dpath);
  var pouch = new PouchDB(dpath);
  pouch.dname = dname;
  var longname = [opts.code, '-', dname].join('');
  var spath = [opts.host, longname].join('/');
  var source = new PouchDB(spath, {
    skip_setup: true
  });
  var stream = new MemoryStream();
  var total = 0;
  var skip = false;
  stream.on('data', function (chunk) {
    if (skip) return;
    total += step;
    var percent = Math.round(parseFloat(1 - (dict.size - total) / dict.size).toFixed(2) * 100); // log('__dumped :', dict.size, total, '%', percent)

    ocounter.innerHTML = 'cloning <b>' + dict.name + '</b> dictionary: ' + percent + '%';
    if (percent > 100) ocounter.textContent = '';
  }).on('error', function (err) {
    log('____stream error', err);
    ocounter.textContent = err;
  }).on('change', function (change) {
    log('____stream change', dname, change);
  }).on('paused', function (info) {
    log('____stream paused', dname, info);
  }).on('active', function (info) {
    log('____stream active', dname, info);
  });
  var batch_size = step * 2;
  return Promise.all([source.dump(stream, {
    batch_size: batch_size,
    live: true,
    retry: true
  })["catch"](function (err) {
    console.log('ERR: DUMP', dname, spath, err.message);
    ocounter.textContent = err.message;
    skip = true;
    throw new Error(err.message);
  }), pouch.load(stream)["catch"](function (err) {
    console.log('ERR: LOAD', dname, dpath, err.message);
    ocounter.textContent = err.message;
    skip = true;
    throw new Error(err.message);
  })]).then(function (res) {
    dbs.push(pouch);
    return true;
  }); // .catch(err=> {
  //   log('ERR: stream dump err', dname, err.message)
  //   // return
  // })
}

function checkConnection(upath, dnames) {
  dbs = [];
  dnames.push('flex');
  dnames = _lodash["default"].uniq(dnames);
  dnames.forEach(function (dname) {
    var dpath = path.resolve(upath, 'pouch', dname);
    var pouch = new PouchDB(dpath);
    pouch.dname = dname;
    dbs.push(pouch);
  });
} // 382, 7


function queryDBs(keys) {
  var qdbs = _lodash["default"].filter(dbs, function (db) {
    return db.dname != 'flex' && db.dname != 'terms';
  });

  return Promise.all(qdbs.map(function (db) {
    return db.allDocs({
      keys: keys,
      include_docs: true
    }).then(function (res) {
      if (!res || !res.rows) throw new Error('no query dbs result');

      var rdocs = _lodash["default"].compact(res.rows.map(function (row) {
        return row.doc;
      }));

      var docs = _lodash["default"].flatten(_lodash["default"].compact(rdocs.map(function (rdoc) {
        return rdoc.docs;
      })));

      docs.forEach(function (doc) {
        doc.dname = db.dname;
      });
      return docs;
    })["catch"](function (err) {
      console.log('ERR queryDBs', err);
    });
  }));
}

function getFlex(keys) {
  var db_flex = _lodash["default"].find(dbs, function (db) {
    return db.dname == 'flex';
  });

  if (!db_flex) return Promise.resolve([]);
  return db_flex.allDocs({
    keys: keys,
    include_docs: true
  }).then(function (res) {
    var rdocs = _lodash["default"].compact(res.rows.map(function (row) {
      return row.doc;
    }));

    var result = [];
    rdocs.forEach(function (fl) {
      fl.docs.forEach(function (doc) {
        result.push(doc);
      });
    });
    return result;
  });
}

function getTerms(wfs) {
  var db_terms = _lodash["default"].find(dbs, function (db) {
    return db.dname == 'terms';
  });

  if (!db_terms) return Promise.resolve([]);
  return db_terms.allDocs({
    keys: wfs,
    include_docs: true
  }).then(function (res) {
    var rdocs = _lodash["default"].compact(res.rows.map(function (row) {
      return row.doc;
    }));

    var docs = _lodash["default"].flatten(rdocs.map(function (rdoc) {
      return rdoc.docs;
    }));

    docs.forEach(function (doc) {
      doc.dname = 'terms';
    });
    return docs;
  });
}

function getDB(wf, dname) {
  var db = _lodash["default"].find(dbs, function (db) {
    return db.dname == dname;
  });

  return db.get(wf);
}

function createDB(upath, docs) {
  var local = 'local';
  var dpath = path.resolve(upath, 'pouch', local);

  var dbloc = _lodash["default"].find(dbs, function (db) {
    return db.dname == local;
  });

  if (dbloc) dbloc.destroy().then(function (res) {
    dbs = _lodash["default"].filter(dbs, function (db) {
      return db.dname != local;
    });
    createLocal(upath, docs);
  });else createLocal(upath, docs);
}

function createLocal(upath, docs) {
  var local = 'local';
  var dpath = path.resolve(upath, 'pouch', local);
  var pouch = new PouchDB(dpath);
  pouch.dname = local;
  dbs.unshift(pouch);
  pouch.bulkDocs(docs).then(function (res) {// log('create-local: BULK-RES', docs)
  });
}

function readDictionary(upath, dname) {
  var local = setLocalDB(upath, dname);
  return local.allDocs({
    include_docs: true,
    startkey: 'α',
    endkey: "\u03C9\uFFF0"
  }).then(function (res) {
    var rdocs = _lodash["default"].compact(res.rows.map(function (row) {
      return row.doc;
    }));

    return rdocs;
  });
}

function updateCurrent(upath, dname, newdocs) {
  determineKey(newdocs);
  return readDictionary(upath, dname).then(function (olddocs) {
    var cleans = [];
    olddocs.forEach(function (rdoc) {
      var newdoc = _lodash["default"].find(newdocs, function (newdoc) {
        return rdoc._id == newdoc._id;
      });

      if (newdoc) newdoc.processed = true, rdoc.docs = newdoc.docs;
      cleans.push(rdoc);
    });

    var news = _lodash["default"].filter(newdocs, function (dict) {
      return !dict.processed;
    });

    cleans.push.apply(cleans, _toConsumableArray(news));
    var local = setLocalDB(upath, dname);
    return local.bulkDocs(cleans).then(function (res) {
      return {
        res: res,
        size: cleans.length
      };
    })["catch"](function (err) {
      console.log('ERR bulkDocs', err);
    });
  });
}

function setLocalDB(upath, dname) {
  var local = _lodash["default"].find(dbs, function (db) {
    return db.dname == dname;
  });

  if (!local) {
    var dpath = path.resolve(upath, 'pouch', dname);
    local = new PouchDB(dpath);
    local.dname = dname;
    dbs.unshift(local);
  }

  return local;
}

function delDictionary(upath, dname) {
  var local = setLocalDB(upath, dname);
  return local.destroy().then(function (response) {
    return true;
  })["catch"](function (err) {
    console.log('Local DB destroy err:', err);
    return false;
  });
}

function determineKey(rdocs) {
  rdocs.forEach(function (rdoc) {
    rdoc.docs.forEach(function (doc) {
      if (doc.key) return;
      doc.dict = (0, _orthos.comb)(doc.rdict);

      if (doc.name) {
        nkeys.forEach(function (nkey) {
          var rekey = new RegExp(nkey + '$');
          var test = doc.dict.replace(rekey, '');
          if (test != doc.dict) doc.key = nkey;
        });
      } else if (doc.verb) {
        vkeys.forEach(function (vkey) {
          var rekey = new RegExp(vkey + '$');
          var test = doc.dict.replace(rekey, '');
          if (test != doc.dict) doc.key = vkey;
        });
      }
    });
  });
}
//# sourceMappingURL=pouch.js.map